use stm32f4xx_hal::{
    adc::{config::SampleTime, Adc},
    gpio::{Analog, Pin},
    pac::ADC1,
};

const FACTOR: f64 = 1.0 / 0.0757856;

pub struct VoltMeter {
    pin: Pin<'C', 0, Analog>,
    adc: Adc<ADC1>,
    threshold: Option<f32>,
}

impl VoltMeter {
    pub fn new(pin: Pin<'C', 0, Analog>, mut adc: Adc<ADC1>, threshold: Option<f32>) -> Self {
        adc.calibrate();
        Self {
            pin,
            adc,
            threshold,
        }
    }

    pub fn voltage(&mut self) -> f64 {
        let sample = self.adc.convert(&self.pin, SampleTime::Cycles_480);
        let mv = self.adc.sample_to_millivolts(sample);

        mv as f64 * FACTOR / 1000.0
    }

    pub fn below_threshold(&mut self) -> bool {
        match self.threshold {
            Some(v) => self.voltage() < v.into(),
            None => false,
        }
    }

    // 0.1V buffer for hysteresis reasons
    pub fn above_threshold(&mut self) -> bool {
        match self.threshold {
            Some(v) => self.voltage() >= (v + 0.1).into(),
            None => true,
        }
    }
}
